@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*-python-*-
 #
-# Copyright (C) 1999-2006 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2009 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -17,7 +17,7 @@ on the local machine to generate ViewVC web pages.
 
 __author__ = "Peter Funk <pf@artcom-gmbh.de>"
 __date__ = "11 November 2001"
-__version__ = "$Revision: 1288 $"
+__version__ = "$Revision: 2143 $"
 __credits__ = """Guido van Rossum, for an excellent programming language.
 Greg Stein, for writing ViewCVS in the first place.
 Ka-Ping Yee, for the GUI code and the framework stolen from pydoc.py.
@@ -54,14 +54,16 @@ import compat; compat.for_standalone()
 
 
 class Options:
-    port = 7467 # default TCP/IP port used for the server
+    port = 49152 # default TCP/IP port used for the server
     start_gui = 0 # No GUI unless requested.
+    daemon = 0 # stay in the foreground by default
     repositories = {} # use default repositories specified in config
     if sys.platform == 'mac':
         host = '127.0.0.1' 
     else:
         host = 'localhost'
     script_alias = 'viewvc'
+    config_file = None
 
 # --- web browser interface: ----------------------------------------------
 
@@ -224,9 +226,16 @@ If this doesn't work, please click on the link above.
             save_stdout = sys.stdout
             save_stderr = sys.stderr
             # For external tools like enscript we also need to redirect
-            # the real stdout file descriptor. (On windows, reassigning the
-            # sys.stdout variable is sufficient because pipe_cmds makes it
-            # the standard output for child processes.)
+            # the real stdout file descriptor.
+            #
+            # FIXME:  This code used to carry the following comment:
+            #
+            #   (On windows, reassigning the sys.stdout variable is sufficient
+            #   because pipe_cmds makes it the standard output for child
+            #   processes.)
+            #
+            # But we no longer use pipe_cmds.  So at the very least, the
+            # comment is stale.  Is the code okay, though?
             if sys.platform != "win32": save_realstdout = os.dup(1) 
             try:
                 try:
@@ -283,10 +292,16 @@ If this doesn't work, please click on the link above.
         # XXX Move this code out of this function.
         # Early loading of configuration here.  Used to
         # allow tinkering with some configuration settings:
-        handle_config()
+        handle_config(options.config_file)
         if options.repositories:
             cfg.general.default_root = "Development"
-            cfg.general.cvs_roots.update(options.repositories)
+            for repo_name in options.repositories.keys():
+                repo_path = os.path.normpath(options.repositories[repo_name])
+                if os.path.exists(os.path.join(repo_path, "CVSROOT",
+                                               "config")):
+                    cfg.general.cvs_roots[repo_name] = repo_path
+                elif os.path.exists(os.path.join(repo_path, "format")):
+                    cfg.general.svn_roots[repo_name] = repo_path
         elif cfg.general.cvs_roots.has_key("Development") and \
              not os.path.isdir(cfg.general.cvs_roots["Development"]):
             sys.stderr.write("*** No repository found. Please use the -r option.\n")
@@ -298,12 +313,11 @@ If this doesn't work, please click on the link above.
         cfg.options.docroot = None
 
         # if cvsnt isn't found, fall back to rcs
-        if (cfg.conf_path is None
-            and cfg.general.cvsnt_exe_path):
+        if (cfg.conf_path is None and cfg.utilities.cvsnt):
           import popen
           cvsnt_works = 0
           try:
-            fp = popen.popen(cfg.general.cvsnt_exe_path, ['--version'], 'rt')
+            fp = popen.popen(cfg.utilities.cvsnt, ['--version'], 'rt')
             try:
               while 1:
                 line = fp.readline()
@@ -318,16 +332,16 @@ If this doesn't work, please click on the link above.
           except:
             pass
           if not cvsnt_works:
-            cfg.cvsnt_exe_path = None
+            cfg.utilities.cvsnt = None
 
         ViewVC_Server(host, port, callback).serve_until_quit()
     except (KeyboardInterrupt, select.error):
         pass
     print 'server stopped'
 
-def handle_config():
+def handle_config(config_file):
   global cfg
-  cfg = viewvc.load_config(CONF_PATHNAME)
+  cfg = viewvc.load_config(config_file or CONF_PATHNAME)
 
 # --- graphical interface: --------------------------------------------------
 
@@ -368,7 +382,7 @@ def gui(host, port):
 
             # Early loading of configuration here.  Used to
             # allow tinkering with configuration settings through the gui:
-            handle_config()
+            handle_config(options.config_file)
             if not LIBRARY_DIR:
                 cfg.options.cvsgraph_conf = "../cgi/cvsgraph.conf.dist"
 
@@ -382,14 +396,6 @@ def gui(host, port):
                 command=self.toggle_use_cvsgraph)
             self.cvsgraph_toggle.pack(side='top', anchor='w')
 
-            # enscript toggle:
-            self.enscript_ivar = Tkinter.IntVar()
-            self.enscript_ivar.set(cfg.options.use_enscript)
-            self.enscript_toggle = Tkinter.Checkbutton(self.options_frm,
-                text="enable enscript (needs binary)", var=self.enscript_ivar,
-                command=self.toggle_use_enscript)
-            self.enscript_toggle.pack(side='top', anchor='w')
-
             # show_subdir_lastmod toggle:
             self.subdirmod_ivar = Tkinter.IntVar()
             self.subdirmod_ivar.set(cfg.options.show_subdir_lastmod)
@@ -415,20 +421,31 @@ def gui(host, port):
                 command=self.toggle_use_localtime)
             self.use_localtime_toggle.pack(side='top', anchor='w')
 
-            # use_pagesize integer var:
-            self.usepagesize_lbl = Tkinter.Label(self.options_frm,
-                text='Paging (number of items per page, 0 disables):')
-            self.usepagesize_lbl.pack(side='top', anchor='w')
-            self.use_pagesize_ivar = Tkinter.IntVar()
-            self.use_pagesize_ivar.set(cfg.options.use_pagesize)
-            self.use_pagesize_entry = Tkinter.Entry(self.options_frm,
-                width=10, textvariable=self.use_pagesize_ivar)
-            self.use_pagesize_entry.bind('<Return>', self.set_use_pagesize)
-            self.use_pagesize_entry.pack(side='top', anchor='w')
+            # log_pagesize integer var:
+            self.log_pagesize_lbl = Tkinter.Label(self.options_frm,
+                text='Paging (number of items per log page, 0 disables):')
+            self.log_pagesize_lbl.pack(side='top', anchor='w')
+            self.log_pagesize_ivar = Tkinter.IntVar()
+            self.log_pagesize_ivar.set(cfg.options.log_pagesize)
+            self.log_pagesize_entry = Tkinter.Entry(self.options_frm,
+                width=10, textvariable=self.log_pagesize_ivar)
+            self.log_pagesize_entry.bind('<Return>', self.set_log_pagesize)
+            self.log_pagesize_entry.pack(side='top', anchor='w')
+
+            # dir_pagesize integer var:
+            self.dir_pagesize_lbl = Tkinter.Label(self.options_frm,
+                text='Paging (number of items per dir page, 0 disables):')
+            self.dir_pagesize_lbl.pack(side='top', anchor='w')
+            self.dir_pagesize_ivar = Tkinter.IntVar()
+            self.dir_pagesize_ivar.set(cfg.options.dir_pagesize)
+            self.dir_pagesize_entry = Tkinter.Entry(self.options_frm,
+                width=10, textvariable=self.dir_pagesize_ivar)
+            self.dir_pagesize_entry.bind('<Return>', self.set_dir_pagesize)
+            self.dir_pagesize_entry.pack(side='top', anchor='w')
 
             # directory view template:
             self.dirtemplate_lbl = Tkinter.Label(self.options_frm,
-                text='Chooose HTML Template for the Directory pages:')
+                text='Choose HTML Template for the Directory pages:')
             self.dirtemplate_lbl.pack(side='top', anchor='w')
             self.dirtemplate_svar = Tkinter.StringVar()
             self.dirtemplate_svar.set(cfg.templates.directory)
@@ -447,7 +464,7 @@ def gui(host, port):
 
             # log view template:
             self.logtemplate_lbl = Tkinter.Label(self.options_frm,
-                text='Chooose HTML Template for the Log pages:')
+                text='Choose HTML Template for the Log pages:')
             self.logtemplate_lbl.pack(side='top', anchor='w')
             self.logtemplate_svar = Tkinter.StringVar()
             self.logtemplate_svar.set(cfg.templates.log)
@@ -500,9 +517,6 @@ def gui(host, port):
         def toggle_use_cvsgraph(self, event=None):
             cfg.options.use_cvsgraph = self.cvsgraph_ivar.get()
 
-        def toggle_use_enscript(self, event=None):
-            cfg.options.use_enscript = self.enscript_ivar.get()
-
         def toggle_use_localtime(self, event=None):
             cfg.options.use_localtime = self.use_localtime_ivar.get()
 
@@ -512,8 +526,11 @@ def gui(host, port):
         def toggle_useresearch(self, event=None):
             cfg.options.use_re_search = self.useresearch_ivar.get()
 
-        def set_use_pagesize(self, event=None):
-            cfg.options.use_pagesize = self.use_pagesize_ivar.get()
+        def set_log_pagesize(self, event=None):
+            cfg.options.log_pagesize = self.log_pagesize_ivar.get()
+
+        def set_dir_pagesize(self, event=None):
+            cfg.options.dir_pagesize = self.dir_pagesize_ivar.get()
 
         def set_templates_log(self, event=None):
             cfg.templates.log = self.logtemplate_svar.get()
@@ -570,8 +587,9 @@ def cli(argv):
     class BadUsage(Exception): pass
 
     try:
-        opts, args = getopt.getopt(argv[1:], 'gp:r:h:s:', 
-            ['gui', 'port=', 'repository=', 'script-alias='])
+        opts, args = getopt.getopt(argv[1:], 'gdc:p:r:h:s:', 
+            ['gui', 'daemon', 'config-file=', 'host=',
+             'port=', 'repository=', 'script-alias='])
         for opt, val in opts:
             if opt in ('-g', '--gui'):
                 options.start_gui = 1
@@ -582,16 +600,29 @@ def cli(argv):
                     options.repositories[symbolic_name] = val
                 else:
                     options.repositories["Development"] = val
+            elif opt in ('-d', '--daemon'):
+                options.daemon = 1
             elif opt in ('-p', '--port'):
                 try:
                     options.port = int(val)
                 except ValueError:
-                    raise BadUsage
+                    raise BadUsage, "Port '%s' is not a valid port number" \
+                          % (val)
             elif opt in ('-h', '--host'):
                 options.host = val
             elif opt in ('-s', '--script-alias'):
                 options.script_alias = \
                     string.join(filter(None, string.split(val, '/')), '/')
+            elif opt in ('-c', '--config-file'):
+                options.config_file = val
+        if options.start_gui and options.config_file:
+            raise BadUsage, "--config-file option is not valid in GUI mode."
+        if not options.start_gui and not options.port:
+            raise BadUsage, "You must supply a valid port, or run in GUI mode."
+        if options.daemon:
+            pid = os.fork()
+            if pid != 0:
+                sys.exit()  
         if options.start_gui:
             gui(options.host, options.port)
             return
@@ -601,42 +632,50 @@ def cli(argv):
                                                 options.script_alias)
             serve(options.host, options.port, ready)
             return
-        raise BadUsage
-    except (getopt.error, BadUsage):
+    except (getopt.error, BadUsage), err:
         cmd = os.path.basename(sys.argv[0])
         port = options.port
         host = options.host
         script_alias = options.script_alias
-        print """ViewVC standalone - a simple standalone HTTP-Server
+        if str(err):
+            sys.stderr.write("ERROR: %s\n\n" % (str(err)))
+        sys.stderr.write("""Usage: %(cmd)s [OPTIONS]
 
-Usage: %(cmd)s [OPTIONS]
+Run a simple, standalone HTTP server configured to serve up ViewVC
+requests.
 
-Available Options:
+Options:
 
--h <host>, --host=<host>:
-    Start the HTTP server listening on <host>.  You need to provide
-    the hostname if you want to access the standalone server from a
-    remote machine.  [default: %(host)s]
+  --config-file=PATH (-c)    Use the file at PATH as the ViewVC configuration
+                             file.  If not specified, ViewVC will try to use
+                             the configuration file in its installation tree;
+                             otherwise, built-in default values are used.
+                             (Not valid in GUI mode.)
+                             
+  --daemon (-d)              Background the server process.
+  
+  --host=HOST (-h)           Start the server listening on HOST.  You need
+                             to provide the hostname if you want to
+                             access the standalone server from a remote
+                             machine.  [default: %(host)s]
 
--p <port>, --port=<port>:
-    Start an HTTP server on the given port.  [default: %(port)d]
+  --port=PORT (-p)           Start the server on the given PORT.
+                             [default: %(port)d]
 
--r <path>, --repository=<path>:
-    Specify a path for a CVS repository.  Repository definitions are
-    typically read from the viewvc.conf file, if available.  This
-    option may be used more than once.
+  --repository=PATH (-r)     Serve up the Subversion or CVS repository located
+                             at PATH.  This option may be used more than once.
 
--s <path>, --script-alias=<path>:
-    Specify the ScriptAlias, the artificial path location that at
-    which ViewVC appears to be located.  For example, if your
-    ScriptAlias is "cgi-bin/viewvc", then ViewVC will appear to be
-    accessible at the URL "http://%(host)s:%(port)s/cgi-bin/viewvc".
-    [default: %(script_alias)s]
-    
--g, --gui:
-    Pop up a graphical interface for serving and testing ViewVC.
-    NOTE: this requires a valid X11 display connection.
-""" % locals()
+  --script-alias=PATH (-s)   Specify the ScriptAlias, the artificial path
+                             location that at which ViewVC appears to be
+                             located.  For example, if your ScriptAlias is
+                             "cgi-bin/viewvc", then ViewVC will be accessible
+                             at "http://%(host)s:%(port)s/cgi-bin/viewvc".
+                             [default: %(script_alias)s]
+  
+  --gui (-g)                 Pop up a graphical interface for serving and
+                             testing ViewVC.  NOTE: this requires a valid
+                             X11 display connection.
+""" % locals())
 
 if __name__ == '__main__':
     options = Options()
