@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2009 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2010 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -274,9 +274,7 @@ class Config:
     """Overlay per-root options for ROOTNAME atop the existing option
     set.  This is a destructive change to the configuration."""
 
-    # We can only deal with this happening once!
-    assert(self.root_options_overlayed == 0)
-    self.root_options_overlayed = 1
+    did_overlay = 0
     
     if not self.conf_path:
       return
@@ -285,7 +283,17 @@ class Config:
       base_section = self._is_allowed_override(self.parser, 'root',
                                                rootname, section)
       if base_section:
+        # We can currently only deal with root overlays happening
+        # once, so check that we've not yet done any overlaying of
+        # per-root options.
+        assert(self.root_options_overlayed == 0)
         self._process_section(self.parser, section, base_section)
+        did_overlay = 1
+
+    # If we actually did any overlaying, remember this fact so we
+    # don't do it again later.
+    if did_overlay:
+      self.root_options_overlayed = 1
 
   def _get_parser_items(self, parser, section):
     """Basically implement ConfigParser.items() for pre-Python-2.3 versions."""
