@@ -1,6 +1,6 @@
 # -*-python-*-
 #
-# Copyright (C) 1999-2009 The ViewCVS Group. All Rights Reserved.
+# Copyright (C) 1999-2010 The ViewCVS Group. All Rights Reserved.
 #
 # By using this file, you agree to the terms and conditions set forth in
 # the LICENSE.html file which can be found at the top level of the ViewVC
@@ -314,14 +314,14 @@ class FileContentsPipe:
 
 
 class BlameSource:
-  def __init__(self, local_url, rev, first_rev):
+  def __init__(self, local_url, rev, first_rev, config_dir):
     self.idx = -1
     self.first_rev = first_rev
     self.blame_data = []
 
     ctx = client.ctx_t()
-    core.svn_config_ensure(None)
-    ctx.config = core.svn_config_get_config(None)
+    core.svn_config_ensure(config_dir)
+    ctx.config = core.svn_config_get_config(config_dir)
     ctx.auth_baton = core.svn_auth_open([])
     try:
       ### TODO: Is this use of FIRST_REV always what we want?  Should we
@@ -376,7 +376,7 @@ class LocalSubversionRepository(vclib.Repository):
     self.auth = authorizer
     self.svn_client_path = utilities.svn or 'svn'
     self.diff_cmd = utilities.diff or 'diff'
-    self.config_dir = config_dir
+    self.config_dir = config_dir or None
 
     # See if this repository is even viewable, authz-wise.
     if not vclib.check_root_access(self):
@@ -567,7 +567,7 @@ class LocalSubversionRepository(vclib.Repository):
     youngest_rev, youngest_path = history[0]
     oldest_rev, oldest_path = history[-1]
     source = BlameSource(_rootpath2url(self.rootpath, path),
-                         youngest_rev, oldest_rev)
+                         youngest_rev, oldest_rev, self.config_dir)
     return source, youngest_rev
 
   def _revinfo(self, rev, include_changed_paths=0):
